/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Soll dem Datensatz mit maximalen Tageswert erzeugen. Daher dieses wert kann
 * nicht mit dem Aggregationsintervall Tag bestimmt werden, erzeugt nur leere
 * Datensaetze mit Werte auf nicht ermittelbar gestellt
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AggregatorSpizenTag extends AbstraktAggregator {
	/**
	 * ATG String Konstante.
	 */
	private static final String ATG_SPITZENTAG = "atg.langZeitDatenSpitzenTag";

	@Override
	public void bearbeiteDatenSatz(final SystemObject objekt,
			final Data datensatz, final long zeitStempel) {
		if (datensatz == null) {
			return;
		}
		final LZZDaten lzz = getLzzDatenForSystemObject(objekt);
		if (lzz == null) {
			return;
		}
		lzz.incrementAnzahlDatenSaetze();
	}

	@Override
	public DataDescription init(final Aspect asp, final ClientDavInterface dav,
			final Collection<SystemObject> objekteLZZ) {

		resetLzzDaten();

		for (final SystemObject lzz : objekteLZZ) {
			final LZZDaten lzzStruct = new LZZDaten();
			final Data data = dav.createData(dav.getDataModel()
					.getAttributeGroup(AggregatorSpizenTag.ATG_SPITZENTAG));
			lzzStruct.setErgebnissDatenSatz(data);
			setLeer(data);
			addLzzDaten(lzz, lzzStruct);
		}

		return new DataDescription(dav.getDataModel().getAttributeGroup(
				AggregatorSpizenTag.ATG_SPITZENTAG), asp);
	}

	/**
	 * Initialsiert einen Datensatz.
	 *
	 * @param data
	 *            DatenSatz
	 */
	@Override
	public void setLeer(final Data data) {
		for (final String element : AbstraktAggregator.FZ_ATTRIB) {
			Data subData = data.getItem(element);
			subData.getTimeValue(AbstraktAggregator.STR_WERT).setMillis(
					AbfrPufferKonstanten.ZEIT_BEGINN);
			Data gueteData = subData.getItem(AbstraktAggregator.STR_GUETE);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_INDEX).set(1.0);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(0);

			subData = data.getItem("q" + element);
			subData.getUnscaledValue(AbstraktAggregator.STR_WERT).set(-1);
			gueteData = subData.getItem(AbstraktAggregator.STR_GUETE);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_INDEX).set(1.0);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(0);
		}
	}

	@Override
	public void erzeugeErgebniss(final SystemObject objekt,
			final long zeitStempelVon, final long zeitStempelBis,
			final boolean ermittelbar) {

		final LZZDaten lzz = getLzzDatenForSystemObject(objekt);
		final Data ergebnissDatenSatz = lzz.getErgebnissDatenSatz();

		if (lzz.getZeitStempelLetztesDS() >= zeitStempelVon) {
			return;
		}

		final ResultData result = new ResultData(objekt, getDdAggregierteLzd(),
				zeitStempelVon, ergebnissDatenSatz, true);
		sendeDatenSatz(objekt, result);

		lzz.setAnzahlDatenSaetze(0);
		lzz.setZeitStempelLetztesDS(zeitStempelVon);
	}

}
